#include "arg_parser.h"

#include <cstdlib>

ArgParser::ArgParser(int argc, char *argv[]) {
    m_good = false;

    m_ip    = "0.0.0.0";
    m_port  = 3142;
    m_save_raw = false;

    for (int i = 1; i < argc; i++) {
        std::string arg = argv[i];

        if (arg == "-i" || arg == "--ip") {
            if (++i >= argc) {
                print_help(argv[0]);
                return;
            }

            m_ip = argv[i];
        }
        else if (arg == "-p" || arg == "--port") {
            if (++i >= argc) {
                print_help(argv[0]);
                return;
            }

            m_port = static_cast<unsigned short>(strtoul(argv[i], nullptr, 10));
        }
        else if (arg == "--save-raw") {
            m_save_raw = true;
        }
        else {
            print_help(argv[0]);
            return;
        }
    }

    m_good = true;
}

bool ArgParser::good() {
    return m_good;
}

std::string ArgParser::get_ip() {
    return m_ip;
}

unsigned short  ArgParser::get_port() {
    return m_port;
}

bool ArgParser::get_save_raw() {
    return m_save_raw;
}

void ArgParser::print_help(const char *argv0) {
    printf("usage: %s [--save-raw] [-i | --ip ip_addr] [-p | --port port]\n", argv0);
}
