#ifndef __auto_pilot_h__
#define __auto_pilot_h__

#include <Eigen/Dense>

#include "exploration_searcher.h"
#include "floormap.h"

#include <slam/misc.h>

#include <SDL2/SDL.h>

#include <cstdint>
#include <list>
#include <vector>

enum class AutoPilotState {
    NAVIGATION  = 0,
    EXPLORING   = 1,

    MAX
};

class AutoPilot {
    public:
        AutoPilot();

        AutoPilot(const AutoPilot &) = delete;
        AutoPilot &operator=(const AutoPilot &) = delete;

        void set_state(AutoPilotState state);
        AutoPilotState get_state();

        Floormap *get_floormap() const;
        void set_floormap(Floormap *floormap);

        void set_power_limit(double power_limit);
        double get_power_limit() const;
        double get_power() const;

        Eigen::Vector2d get_destination() const;
        void set_destination(const Eigen::Vector2d &destination);
        void clear_destination();
        bool has_destination() const;

        void push_lidar_data(const std::vector<LidarData> &lidar_data);

        void get_waypoints(std::vector<Eigen::Vector2d> &waypoints) const;
        void get_frontiers(std::vector<Rect> &frontiers) const;

        bool get_collision() const;
        void set_collision(bool collision);

        void tick();
        void get_speeds(int8_t &left, int8_t &right);
        bool route_good() const;
        bool good() const;
        bool finished() const;

    private:
        AutoPilotState m_state;

        ExplorationSearcher m_exp_search;
        Uint32              m_exp_search_timeout;

        Floormap *m_floormap;
        double m_power_limit;
        double m_power;

        bool m_collision;
        bool m_destination_set;
        Rect m_dest_rect;
        std::list<Eigen::Vector2d> m_waypoints;

        std::vector<Rect> m_frontiers;
        Uint32 m_next_frontier_search;

        // lidar segment count
        int m_front_segments[4]; // a, b, c, d
        int m_side_segments[2]; // left, right

        Uint32 m_next_recalc_route;
        bool m_explorer_fail;

        void recalc_route();
};

#endif //#ifndef __auto_pilot_h__
