#include "box_widget.h"

#include <logging/logging.h>

#include <cassert>

BoxWidget::BoxWidget(BoxWidgetOrientation oritentation) {
    m_orientation = oritentation;
}

size_t BoxWidget::add_child(const std::shared_ptr<Widget> &child) {
    assert(child != nullptr);

    m_children.push_back(child);
    m_children_sizes.push_back({0, 0});

    return m_children.size() - 1;
}

void BoxWidget::remove_child(size_t position) {
    if (position >= m_children.size()) {
        warn("invalid position");
        return;
    }

    m_children.erase(m_children.begin() + position);
    m_children_sizes.erase(m_children_sizes.begin() + position);
}

Widget *BoxWidget::get_child(size_t position) const {
    if (position >= m_children.size())
        return nullptr;

    return m_children[position].get();
}

void BoxWidget::request_size(WidgetSize &size, const WidgetSize &hint) {
    size.width = 0;
    size.height = 0;

    if (m_children.empty())
        return;

    WidgetSize child_hint = hint;
    child_hint.width /= m_children.size();
    child_hint.height /= m_children.size();
    for (size_t i = 0; i < m_children.size(); i++) {
        m_children[i]->request_size(m_children_sizes[i], child_hint);

        if (m_orientation == BoxWidgetOrientation::HORIZONTAL) {
            size.width += m_children_sizes[i].width;
            size.height = std::max(size.height, m_children_sizes[i].height);
        }
        else {
            size.width = std::max(size.width, m_children_sizes[i].width);
            size.height += m_children_sizes[i].height;
        }
    }
}

void BoxWidget::draw(SDL_Renderer *renderer, const WidgetDrawLocation &location) {
    WidgetDrawLocation child_location;
    child_location.x = location.x;
    child_location.y = location.y;

    for (size_t i = 0; i < m_children.size(); i++) {
        if (m_orientation == BoxWidgetOrientation::HORIZONTAL) {
            child_location.size.width = m_children_sizes[i].width;
            child_location.size.height = location.size.height;
        }
        else {
            child_location.size.width = location.size.width;
            child_location.size.height = m_children_sizes[i].height;
        }

        m_children[i]->draw(renderer, child_location);

        if (m_orientation == BoxWidgetOrientation::HORIZONTAL)
            child_location.x += child_location.size.width;
        else
            child_location.y += child_location.size.height;
    }
}

