#ifndef __exploration_searcher_h__
#define __exploration_searcher_h__

#include "floormap.h"

#include <Eigen/Dense>

#include <thread>
#include <mutex>
#include <condition_variable>
#include <vector>
#include <list>

enum class SearchResultType {
    FRONTIER = 0,
    UNVISITED,
    OLD,
};

struct SearchResult {
    Rect                        rect;
    SearchResultType            type;
    int                         sort_value;

    std::list<Eigen::Vector2d>  route;
};

class ExplorationSearcher {
    public:
        ExplorationSearcher();
        ~ExplorationSearcher();

        ExplorationSearcher(const ExplorationSearcher &) = delete;
        ExplorationSearcher &operator=(const ExplorationSearcher &) = delete;

        void set_floormap(Floormap *floormap);
        Floormap *get_floormap() const;

        void start();
        bool done() const;
        void get_results(std::vector<SearchResult> &search_results) const;

    private:
        enum class State {
            DONE,
            START,
            WORKING,
        };

        bool                        m_quit;
        State                       m_state;
        mutable std::mutex          m_lock;
        std::condition_variable     m_cond;
        std::unique_ptr<std::thread> m_thread;
        std::vector<SearchResult>   m_results;

        Floormap                   *m_floormap;

        void worker();
};

#endif // #ifndef __exploration_searcher_h__
