#ifndef __floormap_h__
#define __floormap_h__

#include "lidar_data.h"
#include "imu.h"

#include "slam/point_cloud.h"
#include "slam/imu_data.h"
#include "slam/graph.h"
#include "slam/grid_map.h"

#include <cstdint>
#include <vector>
#include <memory>
#include <thread>
#include <mutex>
#include <queue>
#include <condition_variable>

class Floormap {
    public:
        Floormap();
        ~Floormap();

        void push_lidar_data(uint32_t timestamp, const std::vector<LidarData> &data);
        void push_imu_data(const IMUDatum &datum);

        void get_gm(GridMap &gm);
        void get_hm(Map<int64_t> &hm);
        void get_robot_position(Eigen::Vector2d &vec, double &heading);
        bool get_heading_vel_exceeded();
        bool get_pr_vel_exceeded();

        void worker();

    private:
        bool                    m_quit;
        std::mutex              m_lock;
        std::condition_variable m_cond;
        std::unique_ptr<std::thread> m_thread;

        uint32_t                m_timestamp_prev;
        std::queue<PointCloud>  m_pc_queue;
        IMUData                 m_imu_data;

        Graph                   m_graph;
        GridMap                 m_gm;
        Map<int64_t>            m_hm; // robot position history map


        Eigen::Vector2d         m_robot_position;

        bool                    m_heading_vel_exceeded;
        bool                    m_pr_vel_exceeded;
};

#endif //#ifndef __floormap_h__
