#include "graphics.h"

#include "logging/except.h"
#include "logging/logging.h"

namespace {

const int WINDOW_DEFAULT_WIDTH = 1200;
const int WINDOW_DEFAULT_HEIGHT = 900;
const char *WINDOW_TITLE = "Floor mapping robot client";

}

Graphics::Graphics() {
    int err = SDL_CreateWindowAndRenderer(WINDOW_DEFAULT_WIDTH, WINDOW_DEFAULT_HEIGHT,
            SDL_WINDOW_RESIZABLE, &m_window, &m_renderer);
    if (err != 0)
        throw FM("SDL_CreateWindowAndRenderer");

    SDL_SetWindowTitle(m_window, WINDOW_TITLE);

    if(SDL_SetHint(SDL_HINT_RENDER_SCALE_QUALITY, "1") == SDL_FALSE)
        warn("SDL_SetHint(SDL_HINT_RENDER_SCALE_QUALITY, 1) failed", SDL_GetError());

    SDL_SetRenderDrawColor(m_renderer, 0x00, 0x00, 0x00, 0xff);
    SDL_RenderClear(m_renderer);
    SDL_RenderPresent(m_renderer);
}

Graphics::~Graphics() {
    SDL_DestroyRenderer(m_renderer);
    SDL_DestroyWindow(m_window);
}


void Graphics::draw(Widget *widget) {
    int win_width;
    int win_height;

    SDL_GetWindowSize(m_window, &win_width, &win_height);
    WidgetSize hint = {win_width, win_height};
    WidgetSize size;
    widget->request_size(size, hint);

    if (size.width > win_width || size.height > win_height) {
        win_width = std::max(win_width, size.width);
        win_height = std::max(win_height, size.height);
        SDL_SetWindowSize(m_window, win_width, win_height);
    }

    WidgetDrawLocation location = {0, 0, win_width, win_height};
    widget->draw(m_renderer, location);

    SDL_RenderPresent(m_renderer);
}

