#include "indicator_widget.h"

#include <logging/logging.h>

#include <SDL2/SDL_image.h>

IndicatorWidget::IndicatorWidget(const std::string &indicator) {
    m_img = nullptr;
    m_texture = nullptr;

    m_color = IndicatorWidgetColor::GREEN;

    m_state = false;
    m_timer = 0;

    m_img = IMG_Load(indicator.c_str());
    if (m_img == nullptr) {
        error("IMG_Load()", SDL_GetError());
        return;
    }

    if (m_img->w * static_cast<int>(IndicatorWidgetColor::N_COLORS) != m_img->h) {
        error(indicator, "wrong dimensions");
        SDL_FreeSurface(m_img);
        m_img = nullptr;
    }
}

IndicatorWidget::~IndicatorWidget() {
    if (m_img)
        SDL_FreeSurface(m_img);
    if (m_texture)
        SDL_DestroyTexture(m_texture);
}

void IndicatorWidget::set_color(IndicatorWidgetColor color) {
    m_color = color;
}

IndicatorWidgetColor IndicatorWidget::get_color() const {
    return m_color;
}

void IndicatorWidget::turn_on() {
    m_state = true;
}

void IndicatorWidget::turn_off() {
    m_state = false;
}

void IndicatorWidget::blink() {
    m_timer = SDL_GetTicks() + 1000;
}

void IndicatorWidget::request_size(WidgetSize &size, const WidgetSize &hint) {
    static_cast<void>(hint);

    size.width = m_img->w;
    size.height = m_img->w;
}

void IndicatorWidget::draw(SDL_Renderer *renderer, const WidgetDrawLocation &location) {
    if (m_texture == nullptr && m_img != nullptr) {
        m_texture = SDL_CreateTextureFromSurface(renderer, m_img);
        if (m_texture == nullptr) {
            error("SDL_CreateTextureFromSurface()", SDL_GetError());
            return;
        }
    }

    SDL_Rect src = {0, static_cast<int>(m_color) * m_img->w, m_img->w, m_img->w};
    SDL_Rect dst = {location.x, location.y, m_img->w, m_img->w};

    if (m_state || SDL_GetTicks() < m_timer)
        SDL_RenderCopy(renderer, m_texture, &src, &dst);
}
