#ifndef __main_view_h__
#define __main_view_h__

#include "fixed_widget.h"
#include "indicator_widget.h"
#include "layout_widget.h"
#include "map_widget.h"
#include "radar_widget.h"
#include "scale_widget.h"
#include "streamer_widget.h"

#include <memory>

enum class MainViewMapSize {
    SMALL   = 0,
    NORMAL  = 1,
    LARGE   = 2,
    HUGE    = 3,

    MAX
};

class MainView {
    public:
        MainView();

        MainView(const MainView &) = delete;
        MainView &operator=(const MainView &) = delete;

        StreamerWidget  &get_streamer_widget();
        RadarWidget     &get_radar_widget();
        MapWidget       &get_map_widget();
        ScaleWidget     &get_power_widget();

        IndicatorWidget &get_ap_widget();
        IndicatorWidget &get_navigate_widget();
        IndicatorWidget &get_explorer_widget();
        IndicatorWidget &get_speedset_widget();
        IndicatorWidget &get_battery_widget();
        IndicatorWidget &get_brownout_widget();
        IndicatorWidget &get_camera_widget();
        IndicatorWidget &get_lidar_widget();
        IndicatorWidget &get_cpu_widget();
        IndicatorWidget &get_ram_widget();
        IndicatorWidget &get_collision_widget();
        IndicatorWidget &get_yaw_widget();
        IndicatorWidget &get_pr_widget();

        MainViewMapSize get_map_size() const;
        void set_map_size(MainViewMapSize size);

        Widget *get_widget();

    private:
        std::shared_ptr<StreamerWidget>     m_streamer_widget;
        std::shared_ptr<RadarWidget>        m_radar_widget;
        std::shared_ptr<FixedWidget>        m_map_fixed_widget;
        std::shared_ptr<MapWidget>          m_map_widget;
        std::shared_ptr<ScaleWidget>        m_power_widget;

        std::shared_ptr<IndicatorWidget>    m_ap_widget;
        std::shared_ptr<IndicatorWidget>    m_navigate_widget;
        std::shared_ptr<IndicatorWidget>    m_explorer_widget;
        std::shared_ptr<IndicatorWidget>    m_speedset_widget;
        std::shared_ptr<IndicatorWidget>    m_battery_widget;
        std::shared_ptr<IndicatorWidget>    m_brownout_widget;
        std::shared_ptr<IndicatorWidget>    m_camera_widget;
        std::shared_ptr<IndicatorWidget>    m_lidar_widget;
        std::shared_ptr<IndicatorWidget>    m_cpu_widget;
        std::shared_ptr<IndicatorWidget>    m_ram_widget;
        std::shared_ptr<IndicatorWidget>    m_collision_widget;
        std::shared_ptr<IndicatorWidget>    m_yaw_widget;
        std::shared_ptr<IndicatorWidget>    m_pr_widget;

        std::shared_ptr<LayoutWidget> m_widget;

        MainViewMapSize m_map_size;

        void init_radar_widget();
        void init_map_widget();
        void init_gauges();
};

#endif //#ifndef __main_view_h__
