#ifndef __map_widget_h__
#define __map_widget_h__

#include <Eigen/Dense>

#include "floormap.h"
#include "texture_component.h"
#include "widget.h"

#include <slam/misc.h>

#include <SDL2/SDL.h>

#include <vector>

class MapWidget : public Widget, public TextureComponent {
    public:
        MapWidget(double update_freq = 0.1);

        void set_floormap(Floormap *floormap);
        Floormap *get_floormap() const;

        void set_waypoints(const std::vector<Eigen::Vector2d> &waypoints);
        void set_frontiers(const std::vector<Rect> &frontiers);

        void set_destination(const Eigen::Vector2d &destination);
        void show_destination(bool state);

        int get_scale() const;
        void set_scale(int scale);

        Eigen::Vector2d get_offset();
        void set_offset(const Eigen::Vector2d &offset);

        bool get_show_cursor() const;
        void set_show_cursor(bool show_cursor);

        bool get_follow_robot();
        void set_follow_robot(bool follow_robot);

        virtual void request_size(WidgetSize &size, const WidgetSize &hint);
        virtual void draw(SDL_Renderer *renderer, const WidgetDrawLocation &location);

    private:
        Uint32 m_update_freq;
        Uint32 m_next_update;

        Floormap *m_floormap;
        std::vector<Eigen::Vector2d> m_waypoints;
        std::vector<Rect> m_frontiers;
        bool m_show_destination;
        Eigen::Vector2d m_destination;

        int m_scale;                // px/m
        Eigen::Vector2d m_offset;   // m

        bool    m_show_cursor;
        bool    m_follow_robot;
        Uint32  m_pan_timeout;

        void update_texture(SDL_Renderer *renderer, int width, int height);

        inline Eigen::Vector2d tex_to_map(const Eigen::Vector2i &tex_coord) {
            Eigen::Vector2i tex_center(m_texture_width / 2, m_texture_height / 2);

            return (tex_coord - tex_center).cast<double>() / m_scale + m_offset;
        }

        inline Eigen::Vector2i map_to_tex(const Eigen::Vector2d &map_coord) {
            Eigen::Vector2i tex_center(m_texture_width / 2, m_texture_height / 2);

            return ((map_coord - m_offset) * m_scale).cast<int>() + tex_center;
        }
};

#endif //#ifndef __map_widget_h__
