#include "margin_widget.h"

#include "logging/logging.h"


MarginWidget::MarginWidget(int margin) {
    set_margin(margin);
}

void MarginWidget::set_margin(int margin) {
    if (margin < 0) {
        warn("margin can not be set to a negative value");
        margin = 0;
    }

    m_margin = margin;
}

int MarginWidget::get_margin() const {
    return m_margin;
}

void MarginWidget::request_size(WidgetSize &size, const WidgetSize &hint) {
    if (!m_child) {
        size.width = 0;
        size.height = 0;
        return;
    }

    m_child->request_size(size, hint);
    size.width += 2 * m_margin;
    size.height += 2 * m_margin;
}

void MarginWidget::draw(SDL_Renderer *renderer, const WidgetDrawLocation &location) {
    if (!m_child)
        return;

    WidgetDrawLocation child_location = location;
    child_location.size.width -= 2 * m_margin;
    child_location.size.height -= 2 * m_margin;
    child_location.x += m_margin;
    child_location.y += m_margin;

    m_child->draw(renderer, child_location);
}

