#include "scale_widget.h"

#include <SDL2/SDL.h>

#include <algorithm>

ScaleWidget::ScaleWidget() {
    m_value = 0.0;
    m_limit = 1.0;
}

void ScaleWidget::set_value(double value) {
    m_value = std::clamp(value, 0.0, 1.0);
}

double ScaleWidget::get_value() const {
    return m_value;
}

void ScaleWidget::set_limit(double limit) {
    m_limit = std::clamp(limit, 0.0, 1.0);
}

double ScaleWidget::get_limit() const {
    return m_limit;
}

void ScaleWidget::request_size(WidgetSize &size, const WidgetSize &hint) {
    size = hint;
}

void ScaleWidget::draw(SDL_Renderer *renderer, const WidgetDrawLocation &location) {
    SDL_Rect rect = {location.x, location.y, location.size.width, location.size.height};
    SDL_SetRenderDrawColor(renderer, 0x50, 0x50, 0x50, 0xff);
    SDL_RenderFillRect(renderer, &rect);

    rect.w = std::round(location.size.width * std::min(m_value, m_limit));
    SDL_SetRenderDrawColor(renderer, 0xff, 0xff, 0xff, 0xff);
    SDL_RenderFillRect(renderer, &rect);

    rect.w = std::round(location.size.width * (1.0 - m_limit));
    rect.x = location.x + location.size.width - rect.w;
    SDL_SetRenderDrawColor(renderer, 0xff, 0x00, 0x00, 0xff);
    SDL_RenderFillRect(renderer, &rect);
}
