#include "steering_controller.h"

#include <iostream>
#include <algorithm>

SteeringController::SteeringController() {
    m_acceleration = 0.0;
    m_deceleration = 0.0;
    m_steering = 0.0;

    m_limit = 0.4;
}

void SteeringController::set_acceleration(double value) {
    m_acceleration = std::clamp(value, 0.0, 1.0);
}

void SteeringController::set_deceleration(double value) {
    m_deceleration = std::clamp(value, 0.0, 1.0);
}

void SteeringController::set_steering(double value) {
    m_steering = std::clamp(value, -1.0, 1.0);
}

double SteeringController::get_power() {
    return m_limit * (m_acceleration - m_deceleration);
}

double SteeringController::get_limit() {
    return m_limit;
}

void SteeringController::set_limit(double value) {
    m_limit = std::clamp(value, 0.2, 1.0);
}

void SteeringController::get_speeds(int8_t &left, int8_t &right) {
    double speed_norm = m_limit * (m_acceleration - m_deceleration);
    double left_norm = std::min(1.0, 2 * m_steering + 1);
    double right_norm = std::min(1.0, -2 * m_steering + 1);

    left = std::clamp(static_cast<int>(127 * speed_norm * left_norm), -127, 127);
    right = std::clamp(static_cast<int>(127 * speed_norm * right_norm), -127, 127);
}
