#ifndef __streamer_h__
#define __streamer_h__

extern "C" {
    #include "libavcodec/avcodec.h"
    #include "libavutil/common.h"
    #include "libavutil/imgutils.h"
    #include "libavutil/mathematics.h"
}

#include <cstdint>
#include <stdexcept>
#include <string>
#include <vector>

const int STREAMER_BUFFER_SIZE = 4096;

class StreamerException : public std::runtime_error {
    public:
        explicit StreamerException(const std::string& what_arg) :
            std::runtime_error(what_arg) {}
        explicit StreamerException(const char *what_arg) :
            std::runtime_error(what_arg) {}
};

struct StreamerFrame {
    int width;
    int height;

    const uint8_t *y_plane;
    const uint8_t *u_plane;
    const uint8_t *v_plane;
    int y_pitch;
    int u_pitch;
    int v_pitch;
};

class Streamer {
    public:
        Streamer();
        ~Streamer();

        Streamer(const Streamer &obj) = delete;
        Streamer &operator=(const Streamer &obj) = delete;

        void push_data(const void *buf, size_t size);
        bool get_frame(StreamerFrame &frame);
        void get_size(int &width, int &height);


    private:
        void clean_up();

        uint8_t m_buffer[STREAMER_BUFFER_SIZE + AV_INPUT_BUFFER_PADDING_SIZE];
        AVCodecParserContext *m_parser_ctx;
        AVCodecContext *m_decoder_ctx;
        AVFrame *m_frame0;
        AVFrame *m_frame1;
        AVPacket *m_packet;

        bool m_new_frame;

        int m_width;
        int m_height;
};

#endif //#ifndef __streamer_h__
