#include "texture_component.h"

#include <logging/logging.h>

#include <cassert>

TextureComponent::TextureComponent(Uint32 format, int access) {
    m_format = format;
    m_access = access;

    m_texture_width = -1;
    m_texture_height = -1;
    m_texture = nullptr;
}

TextureComponent::~TextureComponent() {
    if (m_texture != nullptr)
        SDL_DestroyTexture(m_texture);
}

void TextureComponent::allocate_texture(SDL_Renderer *renderer, int width, int height) {
    assert(width > 0 && height > 0);

    if (width == m_texture_width && height == m_texture_height)
        return;

    if (m_texture != nullptr) {
        SDL_DestroyTexture(m_texture);
        m_texture_width = -1;
        m_texture_height = -1;
        m_texture = nullptr;
    }

    m_texture = SDL_CreateTexture(renderer, m_format, m_access, width, height);
    if (m_texture == nullptr) {
        error("SDL_CreateTexture()", SDL_GetError());
        return;
    }

    m_texture_width = width;
    m_texture_height = height;
}

