#ifndef __logging_h__
#define __logging_h__

#include <cerrno>
#include <cstdio>
#include <cstring>
#include <ctime>
#include <string>
#include <unistd.h>

inline std::string __to_string(const std::string &s) {
    return s;
}

inline std::string __to_string(const char *s) {
    return std::string(s);
}

template<typename T>
std::string __to_string(T x) {
    return std::to_string(x);
}

template<typename T>
std::string __make_string(T s0) {
    return __to_string(s0);
}

template<typename T, typename... Args>
std::string __make_string(T s0, Args... args) {
    return __make_string(s0) + " " + __make_string(args...);
}

template<typename... Args>
void __log_msg(const char *type, const char *file, int line, const char *func, Args... args) {
    std::string msg = __make_string(args...);

    const char *start_color = "";
    const char *end_colour  = "\e[39m";
    if (strcmp(type, "error") == 0)
        start_color = "\e[31m";
    else if (strcmp(type, "warn") == 0)
        start_color = "\e[33m";
    else if (strcmp(type, "info") == 0)
        start_color = "\e[32m";
    else if (strcmp(type, "debug") == 0)
        start_color = "\e[34m";

    const char *start_bold = "\e[1m";
    const char *end_bold = "\e[0m";

    if (isatty(STDOUT_FILENO) != 1) {
        start_color = "";
        end_colour = "";
        start_bold = "";
        end_bold = "";
    }

    struct timespec tp;
    clock_gettime(CLOCK_REALTIME, &tp);
    double seconds = tp.tv_sec + (tp.tv_nsec / 1000000000.0);

    fprintf(stdout, "[%.2f] %s%s%s %s%s:%s:%d%s %s\n",
            seconds, start_color, type, end_colour,
            start_bold, file, func, line, end_bold, msg.c_str());
}

#define error( ... )    __log_msg("error",  __FILE__, __LINE__, __FUNCTION__, __VA_ARGS__ )
#define warn( ... )     __log_msg("warn",   __FILE__, __LINE__, __FUNCTION__, __VA_ARGS__ )
#define info( ... )     __log_msg("info",   __FILE__, __LINE__, __FUNCTION__, __VA_ARGS__ )
#define debug( ... )    __log_msg("debug",  __FILE__, __LINE__, __FUNCTION__, __VA_ARGS__ )

#endif //#ifndef __logging_h__

