#ifndef __NETWORK_BASE_H__
#define __NETWORK_BASE_H__


#include <cstdint>
#include <cstdlib>
#include <arpa/inet.h>

// NETWORK_MAX_PAYLOAD_SIZE should not be greater than UINT16_MAX
const int NETWORK_MAX_PAYLOAD_SIZE = 8 * 1024;

// The max size of the internal send buffer
const size_t NETWORK_BUFFER_MAX_SIZE = 16 * 1024 * 1024; // 16 MiB


class Network {
    public:
        Network(const char *ip_addr, short port, bool server = false);
        ~Network();

        void send_pkt(uint8_t type, const void *buf, size_t size);
        bool recv_pkt(uint8_t &type, void *buf, size_t &size);
        bool connected();

    private:
        bool m_connected;
        int m_fd;

        // send buffer
        uint8_t *m_buffer;
        size_t m_buffer_start;
        size_t m_buffer_end;
        size_t m_buffer_size;

        size_t buffer_size() const;
        void buffer_push_back(const void *data, size_t size);
        void buffer_send();
        void buffer_send_all();
};


#endif //#ifndef __NETWORK_BASE_H__
