#ifndef __auto_free_h__
#define __auto_free_h__

template <typename T, typename U>
class AutoFree {
    public:
        AutoFree(T value, U(*free_func)(T ptr)) :
            m_value(value), m_free_func(free_func) {}
        ~AutoFree() {
            m_free_func(m_value);
        }

        AutoFree(const AutoFree &obj) = delete;
        AutoFree &operator=(const AutoFree &obj) = delete;

        operator T() const {
            return m_value;
        }

    private:
        T m_value;
        U(*m_free_func)(T);
};

#endif //#ifndef
