#ifndef __graph_h__
#define __graph_h__

#include "grid_map.h"
#include "local_map.h"
#include "map.h"
#include "rect.h"

#include <vector>
#include <map>

struct Constraint {
    size_t i;
    size_t j;

    Eigen::Matrix3d t;
    double error;
};

class Graph {
    public:
        Graph();

        void write(const std::string &filename) const;
        void insert_pc(const PointCloud &pc);
        void get_gm(GridMap &gm);
        void get_robot_position(Eigen::Vector2d &vec);

    private:
        bool                    m_export_local_maps;
        bool                    m_draw_bounds;
        bool                    m_draw_robot_trail;

        int                     m_count;
        std::vector<LocalMap>   m_local_maps;
        std::vector<Constraint> m_constraints;

        // points to the most recent local map where at this position
        Map<size_t>             m_robot_history;

        // cache for the global grid map (ggm)
        // we also store the ggm version after patching (without the most
        // recent local map). Because generally only the pose of the latest
        // LocalMap changes, hence we can quickly dicard the the latest
        // LocalMap that we added to m_ggm
        GridMap                 m_ggm;
        GridMap                 m_ggm_prev;
        std::map<size_t, Eigen::Vector2d> m_ggm_lm_positions;
        bool                    m_ggm_requires_floodfill;
        bool                    m_ggm_dirty;
        Rect                    m_ggm_patch;

        void finalize();
        void loopclosure(); // acts on the last local_map only
        void optimize();

        void update_ggm_patch(const Rect &patch);
        void ggm_patch();
        void restore_prev();
};

#endif //#ifndef __graph_h__
