#ifndef __imu_data_h__
#define __imu_data_h__

#include <cassert>
#include <cstdint>
#include <climits>
#include <string>
#include <vector>

const size_t HINT_UNKOWN = SIZE_MAX;

struct IMUDatum {
    uint64_t timestamp;

    double heading;
    double pitch;
    double roll;
};

class IMUData {
    public:
        IMUData();

        void read(const std::string &filename);
        void write(const std::string &filename) const;

        const IMUDatum &operator[](size_t i) const {
            assert(i < m_data.size());
            return m_data[i];
        }
        IMUDatum &operator[](size_t i) {
            return const_cast<IMUDatum &>(static_cast<const IMUData &>(*this)[i]);
        }

        void push_back(IMUDatum datum);

        size_t size() const {
            return m_data.size();
        }

        void smooth_heading();

        const IMUDatum &find(uint64_t timestamp, size_t &hint, bool before = true) const;
        IMUDatum interp(uint64_t timestamp, size_t &hint) const;

    private:
        std::vector<IMUDatum> m_data;
        size_t                m_smoothed_index;
        int                   m_rotations; // number of rotations, makes heading continous

        bool find_hint(uint64_t timestamp, size_t &hint) const;
        void find_bin_search(uint64_t timestamp, size_t &hint) const;
};

#endif //#ifndef __imu_data_h__
