#include "local_map.h"
#include "passes.h"

#include <cassert>

using namespace Eigen;

LocalMap::LocalMap(int start_index, int end_index, const Eigen::Matrix3d &pose,
        double cell_size, size_t grid_size) :
    m_start_index(start_index),
    m_end_index(end_index),
    m_pose(pose),
    m_current_pose(Matrix3d::Identity()),
    m_map(cell_size, grid_size) {}


void LocalMap::insert_pc(const PointCloud &pc) {
    assert(pc.size() > 0);

    m_point_clouds.push_back(pc);
    if (m_point_clouds.size() > 0)
        align_pc_to_gm2(&m_map, &m_point_clouds.back(), m_current_pose);
    m_point_clouds.back().transform(m_current_pose);

    if (!pc.get_skip() || m_point_clouds.size() == 0) {
        pass_gm_insert_pc(m_map, m_point_clouds.back());
        m_map.push_back_robot(m_point_clouds.back().get_center());
    }
}

const Eigen::Matrix3d &LocalMap::get_global_pose() const {
    return m_pose;
}

void LocalMap::set_global_pose(Matrix3d &pose) {
    m_pose = pose;
}

const Matrix3d &LocalMap::get_current_pose() const {
    return m_current_pose;
}

int LocalMap::get_start_index() const {
    return m_start_index;
}

int LocalMap::get_end_index() const {
    return m_end_index;
}

const GridMap &LocalMap::get_gm() const {
    return m_map;
}

GridMap &LocalMap::get_gm() {
    return const_cast<GridMap &>(static_cast<const LocalMap &>(*this).get_gm());
}

const std::vector<PointCloud> &LocalMap::get_point_clouds() const {
    return m_point_clouds;
}
