#ifndef __local_map_h__
#define __local_map_h__

#include <Eigen/Dense>

#include "grid_map.h"
#include "point_cloud.h"

#include <vector>

class LocalMap {
    public:
        LocalMap(int start_index, int end_index, const Eigen::Matrix3d &pose,
                double cell_size, size_t grid_size);

        void insert_pc(const PointCloud &pc);

        const Eigen::Matrix3d &get_global_pose() const;
        void set_global_pose(Eigen::Matrix3d &pose);

        const Eigen::Matrix3d &get_current_pose() const;

        int get_start_index() const;
        int get_end_index() const;

        const GridMap &get_gm() const;
        GridMap &get_gm();

        const std::vector<PointCloud> &get_point_clouds() const;

    private:
        // start and stop frame
        int                     m_start_index;
        int                     m_end_index;

        Eigen::Matrix3d         m_pose;
        Eigen::Matrix3d         m_current_pose;
        GridMap                 m_map;

        // Point clouds in the gridmap, with local transform applied
        std::vector<PointCloud> m_point_clouds;
};

#endif //#ifndef __local_map_h__
