#ifndef __point_cloud_h__
#define __point_cloud_h__

#include <Eigen/Dense>

#include <cassert>
#include <vector>
#include <string>

struct PointCloudDatum {
    uint64_t timestamp;

    Eigen::Vector2d data;
};

class PointCloud {
    public:
        PointCloud(Eigen::Vector2d center = Eigen::Vector2d::Zero()) :
            m_center(center)
        {
            m_data.reserve(2048);
            m_skip = false;
        }

        void read(const std::string &filename);
        void write(const std::string &filename) const;

        void calc_timestamps(uint64_t t0, uint64_t t1);

        Eigen::Vector2d get_center() const {
            return m_center;
        }

        void reserve(size_t size) {
            m_data.reserve(size);
        }

        void push_back(const PointCloudDatum &datum) {
            m_data.push_back(datum);
        }

        size_t size() const {
            return m_data.size();
        }

        void clear(Eigen::Vector2d center = Eigen::Vector2d::Zero());

        const PointCloudDatum &operator[](size_t i) const {
            assert(i < m_data.size());
            return m_data[i];
        }

        PointCloudDatum &operator[](size_t i) {
            return const_cast<PointCloudDatum &>(static_cast<const PointCloud &>(*this)[i]);
        }

        void transform(const Eigen::Matrix3d &t);

        void set_skip(bool skip);
        bool get_skip() const;

    private:
        Eigen::Vector2d                 m_center;
        std::vector<PointCloudDatum>    m_data;
        // set to true when the data is bad
        bool                            m_skip;
};

#endif //#ifndef __point_cloud_h__
