#ifndef __rect_h__
#define __rect_h__

#include <Eigen/Dense>

struct Rect {
    Rect() : pos(0.0, 0.0), dim(0.0, 0.0) {};

    Rect(const Eigen::Vector2d &_pos, const Eigen::Vector2d &_dim) :
        pos(_pos), dim(_dim) {}

    Rect(double xmin, double xmax, double ymin, double ymax) {
        pos = Eigen::Vector2d(xmin, ymin);
        dim = Eigen::Vector2d(xmax, ymax) - pos;
    }

    double get_xmin() const {
        return pos[0];
    }

    double get_xmax() const {
        return pos[0] + dim[0];
    }

    double get_ymin() const {
        return pos[1];
    }

    double get_ymax() const {
        return pos[1] + dim[1];
    }

    Eigen::Vector2d get_center() const {
        return pos + dim / 2;
    }

    inline bool inside(const Eigen::Vector2d &vec) const {
        return (vec[0] >= pos[0] && vec[0] < (pos[0] + dim[0])) &&
            (vec[1] >= pos[1] && vec[1] < (pos[1] + dim[1]));
    }

    inline bool overlaps(const Rect &r) {
        return (pos[0] + dim[0] >= r.pos[0]) && (pos[0] <= r.pos[0] + r.dim[0]) &&
               (pos[1] + dim[1] >= r.pos[1]) && (pos[1] <= r.pos[1] + r.dim[1]);
    }

    Eigen::Vector2d pos; // left top corner
    Eigen::Vector2d dim; // width and height
};

#endif //#ifndef __rect_h__
