#ifndef __bno055_h__
#define __bno055_h__

#include <cstdint>
#include <stdexcept>
#include <string>

class BNO055Exception : public std::runtime_error {
    public:
        explicit BNO055Exception(const std::string& what_arg) :
            std::runtime_error(what_arg) {}
        explicit BNO055Exception(const char *what_arg) :
            std::runtime_error(what_arg) {}
};

struct BNO055Event {
    double eul[3];
    double lia[3];
};

class BNO055 {
    public:
        BNO055(const char *i2c_dev, uint32_t addr = 0x28);
        ~BNO055();

        BNO055(const BNO055 &) = delete;
        BNO055 &operator=(const BNO055 &) = delete;

        void set_offset(double pitch_off, double roll_off);

        bool ready();
        bool calibrated();
        void get(BNO055Event &event);

    private:
        void    i2c_send(uint8_t reg, uint8_t data);
        uint8_t i2c_recv(uint8_t reg);
        void    i2c_recv_vec(uint8_t reg_start, int16_t *data, size_t len);

        void delay(uint32_t msec);

        void init();
        void reset();

        int m_fd;
        double m_pitch_off;
        double m_roll_off;
};

#endif //#ifndef __bno055_h__
