#include "heartbeat.h"

namespace {

std::chrono::milliseconds HEARTBEAT_TIMEOUT(200);
std::chrono::milliseconds HEARTBEAT_DEBOUNCE_TIME(200);

enum {
    ALIVE = 0,
    DEAD = 1,
    DEBOUNCE = 2
};

}

Heartbeat::Heartbeat() {
    m_state = ALIVE;

    beat();
}

void Heartbeat::beat() {
    auto t = std::chrono::high_resolution_clock::now();

    m_next_beat = t + HEARTBEAT_TIMEOUT;
}

bool Heartbeat::should_stop() {
    auto t = std::chrono::high_resolution_clock::now();

    switch (m_state) {
        case ALIVE:
            if (t > m_next_beat) {
                m_state = DEAD;
                return true;
            }
            break;

        case DEAD:
            if (t < m_next_beat) {
                m_next_debounce = t + HEARTBEAT_DEBOUNCE_TIME;
                m_state = DEBOUNCE;
            }
            break;

        case DEBOUNCE:
            if (t > m_next_beat)
                m_state = DEAD;
            else if (t > m_next_debounce)
                m_state = ALIVE;
            break;
    }

    return false;
}

bool Heartbeat::alive() {
    return m_state == ALIVE;
}
