#include "rplidar.h"

using namespace rp::standalone::rplidar;

RPLidar::RPLidar(const char *dev) {
    m_driver = nullptr;

    m_driver = RPlidarDriver::CreateDriver(DRIVER_TYPE_SERIALPORT);
    if (m_driver == nullptr)
        throw RPLidarException("Failed to create Driver");

     if (! IS_OK(m_driver->connect(dev, 256000))) {
        clean_up();
        throw RPLidarException("Failed to connect to lidar device");
    }

    rplidar_response_device_info_t devinfo;
    if (! IS_OK(m_driver->getDeviceInfo(devinfo))) {
        clean_up();
        throw RPLidarException("Failed to get device info");
    }

    rplidar_response_device_health_t healthinfo;
    if (! IS_OK(m_driver->getHealth(healthinfo))) {
        clean_up();
        throw RPLidarException("Failed to get health status");
    }

    if (healthinfo.status == RPLIDAR_STATUS_ERROR) {
        clean_up();
        throw RPLidarException("Error status set on lidar");
    }

    m_driver->startMotor();
    m_driver->startScan(0, 1);
}

RPLidar::~RPLidar() {
    if (m_driver)
        m_driver->stopMotor();

    clean_up();
}

void RPLidar::get_points(RPLidarPoint *points, size_t &size) {
    if (size < RP_LIDAR_MAX_POINTS)
        throw RPLidarException("points array not large enough");

    rplidar_response_measurement_node_hq_t nodes[RP_LIDAR_MAX_POINTS];
    size_t count = RP_LIDAR_MAX_POINTS;


    if (! IS_OK(m_driver->grabScanDataHq(nodes, count)))
        throw RPLidarException("grabScanDataHq(): Failed");
    m_driver->ascendScanData(nodes, count);

    for (size_t i = 0; i < count; i++) {
        points[i].angle = nodes[i].angle_z_q14;
        points[i].distance = std::min((uint32_t)UINT16_MAX, nodes[i].dist_mm_q2 / 4);
    }
    size = count;
}

void RPLidar::clean_up() {
    RPlidarDriver::DisposeDriver(m_driver);
    m_driver = nullptr;
}
